@@ -1,4 +1,5 @@
 import abc
+import functools
 
 from copy import deepcopy
 from fractions import Fraction
@@ -33,14 +34,52 @@ from sympy.core.function import _coeff_isneg
 from . import converters
 from .definitions import ALLOWED_VARIABLES, EQN_DEFINITIONS, SUPPLEMENTAL_EQUATIONS
 
+try:
+    from ._version import version as __version__
+except ModuleNotFoundError:
+    __version__ = ""
+
+
 #: dictionary of constants
 CONSTANTS = {"G": G, "c": c, "pi": pi}
 
 
+def function_call_signature(func):
+    """
+    Decorator function to record the call signature of a function. This is
+    based on https://www.geeksforgeeks.org/python-get-function-signature/.
+    """
+
+    # get argument variable names
+    argnames = func.__code__.co_varnames[: func.__code__.co_argcount]
+
+    # get called function name
+    fname = func.__name__
+
+    @functools.wraps(func)
+    def wrapper(*args, **kwargs):
+        # generate string with function call signature
+        funccall = fname + "("
+        for item in list(zip(argnames, args[: len(argnames)])) + list(kwargs.items()):
+            funccall += f"{item[0]}="
+            funccall += f'"{item[1]}"' if isinstance(item[1], str) else f"{item[1]}"
+            funccall += ", "
+        funccall = funccall.strip().rstrip(",") + ")"
+
+        # pass signature to function (which should capture it in some way)
+        kwargs["function_call"] = funccall
+        res = func(*args, **kwargs)
+        kwargs.pop("function_call")
+        return res
+
+    return wrapper
+
+
 def constfunc(name):
     return CONSTANTS[name]
 
 
+@function_call_signature
 def equations(equation, **kwargs):
     """
     This function generates a :class:`~cweqgen.equations.EquationBase` class
@@ -208,6 +247,9 @@ class EquationBase:
         # BibTeX for reference (from ADS)
         self.reference_bibtex = kwargs.pop("reference_bibtex", None)
 
+        # get function call for equations function
+        self.equations_call = kwargs.get("function_call", None)
+
         # make sure Sympy version of equation is generated
         _ = self.sympy_var
         _ = self.constant
@@ -282,9 +324,15 @@ class EquationBase:
 
                     # check value has correct sign
                     if ALLOWED_VARIABLES[key]["sign"] is not None:
+                        varray = np.asarray(value.value)
+                        try:
+                            thresh = len(varray)
+                        except TypeError:
+                            thresh = None
+
                         strrep = (
                             "np.array("
-                            + np.array2string(np.asarray(value.value), separator=", ")
+                            + np.array2string(varray, separator=", ", threshold=thresh)
                             + ")"
                         )
 
@@ -375,7 +423,7 @@ class EquationBase:
         else:
             return fiducial
 
-    def equation(self, displaytype="string", **latexkwargs):
+    def equation(self, displaytype="string", nocomment=False, **latexkwargs):
         """
         Generate the LaTeX string for the equation.
 
@@ -387,6 +435,10 @@ class EquationBase:
             this will show as a formatted LaTeX equation. Alternatively, set to
             "matplotlib" to have the output returned as a Matplotlib figure
             object containing the equation.
+        nocomment: bool
+            By default the output LaTeX string will contain a comment line
+            giving the cweqgen version and call signature from the equations
+            function. To turn this off, set this argument to True.
         latexkwargs: dict
             Keyword parameters that can be passed to the
             :func:`sympy.printing.latex.latex` function. By default the
@@ -428,7 +480,14 @@ class EquationBase:
         if displaytype.lower() == "matplotlib":
             return EquationLaTeXToImage(latex_equation)
         else:
-            return EquationLaTeXString(latex_equation)
+            # add LaTeX comment to string with cweqgen version and equations call
+            comment = None
+            if not nocomment:
+                comment = f"equation generated with cweqgen v{__version__}"
+                if self.equations_call is not None:
+                    comment += f":\n  {self.equations_call}"
+
+            return EquationLaTeXString(latex_equation, comment=comment)
 
     @property
     def eqn(self):
@@ -447,7 +506,9 @@ class EquationBase:
     def _repr_latex_(self):
         return "$" + str(self.equation()) + "$"
 
-    def fiducial_equation(self, dp=2, brackets="()", displaytype="string", **kwargs):
+    def fiducial_equation(
+        self, dp=2, brackets="()", displaytype="string", nocomment=False, **kwargs
+    ):
         """
         Generate the LaTeX string for the equation inserting in fiducial values.
 
@@ -464,6 +525,10 @@ class EquationBase:
             this will show as a formatted LaTeX equation. Alternatively, set to
             "matplotlib" to have the output returned as a Matplotlib figure
             object containing the equation.
+        nocomment: bool
+            By default the output LaTeX string will contain a comment line
+            giving the cweqgen version and call signature from the equations
+            function. To turn this off, set this argument to True.
         """
 
         latex_equation = self.latex_name + " = "
@@ -543,7 +608,14 @@ class EquationBase:
         if displaytype.lower() == "matplotlib":
             return EquationLaTeXToImage("$" + latex_equation + "$")
         else:
-            return EquationLaTeXString(latex_equation)
+            comment = None
+            if not nocomment:
+                # add LaTeX comment to string with cweqgen version and equations call
+                comment = f"equation generated with cweqgen v{__version__}"
+                if self.equations_call is not None:
+                    comment += f":\n  {self.equations_call}"
+
+            return EquationLaTeXString(latex_equation, comment=comment)
 
     def evaluate(self, **kwargs):
         """
@@ -655,7 +727,7 @@ class EquationBase:
 
         newfiducial = newkwargs.pop("default_fiducial_values")
         newfiducial.pop(newvar)
-        newfiducial[self.equation_name] = curval
+        newfiducial[self.variable] = curval
 
         if isinstance(equal, EquationBase):
             newfiducial.update(equal.kwargs["default_fiducial_values"])
@@ -737,9 +809,6 @@ class EquationBase:
 
         return EquationBase(**newkwargs)
 
-    def __str__(self):
-        return str(self.equation())
-
     @property
     def sympy_const(self):
         """
@@ -753,6 +822,12 @@ class EquationBase:
 
             for arg in self.sympy.rhs.args:
                 if arg.is_constant():
+                    if (
+                        arg.is_negative
+                        and ALLOWED_VARIABLES[self.variable]["sign"] == ">= 0"
+                    ):
+                        arg *= -1  # flip sign
+
                     self._sympy_const *= arg
                 else:
                     if isinstance(arg, Symbol):
@@ -972,6 +1047,8 @@ class EquationBase:
             requested variable to parameterise it.
         """
 
+        neweqn = None
+
         # the conversion for frequency equations to loop through
         chain = [
             ("rotationperiod", equations("rotationperiod_to_angularrotationfrequency")),
@@ -984,26 +1061,6 @@ class EquationBase:
             ("rotationfrequency", equations("rotationfrequency_to_period")),
         ]
 
-        # find the index of the end point
-        endidx = ([link[0] for link in chain]).index(end)
-
-        neweqn = None
-        # find the starting conversion equation
-        for i in range(len(chain)):
-            if chain[i][0] in starteqn.var_names:
-                break
-
-        while True:
-            # loop over conversions until finished
-            if neweqn is None:
-                neweqn = starteqn.substitute(chain[i][1])
-            else:
-                neweqn = neweqn.substitute(chain[i][1])
-
-            i = (i + 1) % len(chain)
-            if i == endidx:
-                break
-
         # the conversion for frequency derivative equations to loop through
         chaindot = [
             (
@@ -1034,23 +1091,42 @@ class EquationBase:
         ]
 
         # find the index of the end point
-        endidxdot = ([link[0] for link in chaindot]).index(end)
+        endidx = ([link[0] for link in chain]).index(end)
 
         # find the starting conversion equation
-        for i in range(len(chaindot)):
-            if chaindot[i][1] in starteqn.var_names:
+        for i in range(len(chain)):
+            if chain[i][0] in starteqn.var_names:
                 break
 
         while True:
             # loop over conversions until finished
-            for eqn in chaindot[i][2]:
-                if neweqn is None and eqn.variable in starteqn.var_names:
-                    neweqn = starteqn.substitute(eqn)
-                elif eqn.variable in neweqn.var_names:
-                    neweqn = neweqn.substitute(eqn)
+            if neweqn is None:
+                neweqn = starteqn.substitute(chain[i][1])
+            else:
+                neweqn = neweqn.substitute(chain[i][1])
 
-            i = (i + 1) % len(chaindot)
-            if i == endidxdot:
+            # loop to same point as frequency parameter
+            endidxdot = (i + 1) % len(chain)
+
+            # find the starting conversion equation
+            for j in range(len(chaindot)):
+                if chaindot[j][1] in starteqn.var_names:
+                    break
+
+            while True:
+                # loop over conversions until finished
+                for eqn in chaindot[j][2]:
+                    if neweqn is None and eqn.variable in starteqn.var_names:
+                        neweqn = starteqn.substitute(eqn)
+                    elif eqn.variable in neweqn.var_names:
+                        neweqn = neweqn.substitute(eqn)
+
+                j = (j + 1) % len(chaindot)
+                if j == endidxdot:
+                    break
+
+            i = (i + 1) % len(chain)
+            if i == endidx:
                 break
 
         return neweqn
@@ -1100,7 +1176,7 @@ class EquationBase:
 
 
 class EquationLaTeXString:
-    def __init__(self, latexstring):
+    def __init__(self, latexstring, comment=None):
         """
         Class to hold a LaTeX equation string. It has a _repr_latex_ method to
         hook into the Jupyter notebook rich display system
@@ -1111,15 +1187,28 @@ class EquationLaTeXString:
         ----------
         latexstring: str
             The LaTeX string defining the equation.
+        comment: str
+            A LaTeX comment (% will be prepended to any lines)
         """
 
         self.text = str(latexstring)
+        self.comment = comment
 
     def __repr__(self):
-        return self.text
+        if self.comment is None:
+            return self.text
+        else:
+            output = "".join(f"% {line}\n" for line in self.comment.split("\n"))
+            output += self.text
+            return output
 
     def __str__(self):
-        return self.text
+        if self.comment is None:
+            return self.text
+        else:
+            output = "".join(f"% {line}\n" for line in self.comment.split("\n"))
+            output += self.text
+            return output
 
     def _repr_latex_(self):
         return "$" + self.text + "$"
